@@ -2,7 +2,9 @@ import argparse
 import csv
 import io
 import logging
+import random
 import shutil
+import tarfile
 import threading
 import time
 import tkinter as tk
@@ -153,6 +155,9 @@ class ArchiveBase:
             logger.debug(f"cache {len(yet)} files. : {self.cache.keys()}")
 
     def __getitem__(self, i):
+        if len(self) == 0:
+            return None, None
+
         i = self.in_range(i)
 
         if self.cache.get(i) is not None:
@@ -244,6 +249,10 @@ class DirectoryArchive(ArchiveBase):
         else:
             return "", None
 
+    def random_select(self):
+        i = random.randrange(len(self))
+        return self.getitem(i)
+
 
 class ZipArchive(ArchiveBase):
     def __init__(self, file_path, data=None):
@@ -518,6 +527,70 @@ class PdfArchive(ArchiveBase):
         return file_name, image
 
 
+class TarArchive(ArchiveBase):
+    def __init__(self, file_path, data=None):
+        if "tarfile" not in "globals":
+            global tarfile
+            import tarfile
+        super().__init__()
+        self.multi_read = True
+        self.open(file_path, data)
+        self.start_preload()
+
+    def open(self, file_path, data=None):
+        logger.debug("called")
+        self.file_path = file_path
+        self.data = data
+        self.file_list = []
+
+        logger.debug("to byte")
+        if self.data is not None:
+            self.data.seek(0)
+        fp = self.file_path if data is None else self.data
+
+        logger.debug("open tar")
+        with tarfile.open(fp) as f:
+            self.file_list = f.getnames()
+
+        logger.debug("open tar")
+        self.sort_file_list()
+        self.filtering_file_list()
+        logger.debug(self.file_list)
+
+    def getitems(self, start, end):
+        end += 1
+        logger.debug("called")
+        logger.debug(f"start, end = {start}, {end}")
+
+        file_names = self.file_list[start:end]
+        logger.debug(f"file_names = {file_names}")
+        fp = self.file_path if self.data is None else self.data
+
+        with tarfile.open(fp) as f:
+            file_bytes = [io.BytesIO(f.extractfile(name).read()) for name in file_names]
+
+        logger.debug(f"return. {len(file_names)}")
+        return file_names, file_bytes
+
+    def getitem(self, i):
+        logger.debug("__getitem__")
+        file_name = ""
+        file_byte = None
+        logger.debug("to byte")
+        fp = self.file_path if self.data is None else self.data
+        logger.debug("read file")
+        if 0 <= i < len(self):
+            with tarfile.open(fp) as f:
+                file_name = Path(self.file_list[i])
+                file_byte = f.extractfile(self.file_list[i]).read()
+
+        logger.debug(f"i={i}")
+        logger.debug(self.file_list[i])
+        logger.debug(file_name)
+        logger.debug("return")
+        return file_name, io.BytesIO(file_byte)
+
+
 class ArchiveTree:
     def __init__(self):
         self.root = []
@@ -877,6 +950,9 @@ class MoveFile:
             self.ret = False
             return
 
+        if file_path.resolve() == Path(to / file_path.name).resolve():
+            return
+
         to = to / file_path.name
         if to.exists() and not messagebox.askokcancel(
             "File exists.", "File exists. Overwrite?"
@@ -884,6 +960,7 @@ class MoveFile:
             logger.debug("Do not overwrite.")
             self.ret = False
             return
+        
 
         logger.debug(f"Move {file_path} -> {to}")
 
@@ -930,30 +1007,32 @@ DoublePage  = d
 # You can use repetition for NextPage and PrevPage.
 # For example, 2h means goto next 2 page, type 100h go to next 100 page.
 # If you want to reset number, type <Esc>, <Ctrl+[> or simply <[>
-NextPage    = h
-PrevPage    = l
+NextPage     = h
+PrevPage     = l
 
-NextArchive = j
-PrevArchive = k
+NextArchive  = j
+PrevArchive  = k
 
-Head        = g
-Tail        = G
+Head         = g
+Tail         = G
 
 
-FitNone     = N
-FitWidth    = W
-FitHeight   = H
-FitBoth     = B
+FitNone      = N
+FitWidth     = W
+FitHeight    = H
+FitBoth      = B
 
-PageOrder   = o
+PageOrder    = o
 
 
-TrashFile   = Delete
-MoveFile    = m
+TrashFile    = Delete
+MoveFile     = m
 
-Quit        = q
-FullScreen  = f
-Reload      = r
+Quit         = q
+FullScreen   = f
+Reload       = r
+
+RandomSelect = n
 
 
 [MoveToList]
@@ -1027,6 +1106,8 @@ class SaltViewer(tk.Tk):
         logger.debug("tk.Tk init")
         super().__init__()
 
+        self.file_path = None
+
         self.archive = None
 
         # this directory is the parent of first file_path that passed
@@ -1060,6 +1141,7 @@ class SaltViewer(tk.Tk):
             "FullScreen": self.full_screen,
             "Head": self.head,
             "Tail": self.tail,
+            "RandomSelect": self.random_select,
         }
 
         logger.debug("style")
@@ -1081,6 +1163,14 @@ class SaltViewer(tk.Tk):
 
         self.load_config(args)
 
+    def random_select(self, event):
+        logger.debug("random_select called")
+        self._load_root_dir_thread(self.file_path)
+        self.tree.reset()
+        self.archive.close()
+        self.archive = None
+        self.open(*self.root_dir.random_select())
+
     def move_file(self, event):
 
         fullscreen = self.attributes("-fullscreen")
@@ -1117,7 +1207,7 @@ class SaltViewer(tk.Tk):
 
         self.root_dir.remove(file_path)
         self.root_dir.cache = {}
-        next_file_path = self.root_dir.next()[0]
+        next_file_path = self.root_dir.current()[0]
 
         self.attributes("-fullscreen", fullscreen)
         logger.debug(f"open {next_file_path}")
@@ -1423,8 +1513,17 @@ class SaltViewer(tk.Tk):
     def open(self, file_path, data=None):
         if self.archive is not None:
             self.archive.stop = True
+        if self.root_dir is None:
+            logger.debug("self.root_dir is None load directory")
+            self._load_root_dir(file_path)
+        self.file_path = file_path
+
         self.archive = self.open_archive(file_path, data)
         file_path, data = self.archive.current()
+        logger.debug(f"file_path={file_path}")
+        if file_path is None and data is None:
+            logger.debug("file may be empty.")
+            self.destroy()
         image = self.open_file(file_path, data)
         logger.debug("-------------------------------------")
         logger.debug("open")
@@ -1433,9 +1532,19 @@ class SaltViewer(tk.Tk):
         self.image.display(image)
         return image
 
+    def _load_root_dir_thread(self, file_path):
+        if self.root_dir is None:
+            self.root_dir = DirectoryArchive(file_path)
+
+    def _load_root_dir(self, file_path):
+        logger.debug("start load_root dir")
+        t = threading.Thread(target=self._load_root_dir_thread, args=(file_path,))
+        t.start()
+
     def open_archive(self, file_path, data=None):
         logger.debug("called")
         suffix = Path(file_path).suffix.lower()
+
         if suffix == ".zip":
             logger.debug("zip")
             archive = ZipArchive(file_path, data)
@@ -1448,6 +1557,9 @@ class SaltViewer(tk.Tk):
         elif suffix == ".pdf":
             logger.debug("pdf")
             archive = PdfArchive(file_path, data)
+        elif suffix in [".tar", ".gz"]:
+            logger.debug("pdf")
+            archive = TarArchive(file_path, data)
         else:
             logger.debug("directory")
             archive = DirectoryArchive(file_path, data)
@@ -1476,6 +1588,9 @@ class SaltViewer(tk.Tk):
             title = f"{self.archive.file_path}/" + title
         page = f"({self.archive.i+1}/{len(self.archive)}):"
 
+        logger.debug(page)
+        logger.debug(title)
+
         self.title(page + title)
         self.image.title = title
 
